import copy
import os
import pickle
import random
import sys
from datetime import datetime, timedelta
from operator import *
from typing import Any, List, Optional, Sequence, Text, Tuple, Union, overload

import pygame
import pygame_menu
from pygame.locals import *

from primaryschool.dirs import *
from primaryschool.locale import _, sys_lang_code
from primaryschool.resource import (default_font, default_font_path,
                                    get_default_font, get_font_path,
                                    get_sys_font_by_lang_code)
from primaryschool.subjects import *
from primaryschool.subjects._abc_ import GameBase
from primaryschool.subjects._common_.keycode import PsKeyCode


class ShootingWave():
    def __init__(self, _manager):

        self.manager = _manager
        self.shtbase = self.manager.shtbase
        self.ps = self.shtbase.ps
        self.intercept_interval = self.manager.intercept_interval
        self.surface = self.shtbase.surface
        self.w_height = self.shtbase.w_height
        self.w_height_of_2 = self.shtbase.w_height_of_2
        self.w_width_of_2 = self.shtbase.w_width_of_2
        self.w_centrex_y = self.shtbase.w_centrex_y
        self.color = (0, 255, 0, 20)
        self.width = 5

        self.max_radius = self.get_max_radius()

    def set_color(self, color):
        self.color = color

    def get_max_radius(self):
        return (self.w_height**2 + self.w_width_of_2**2)**0.5

    def set_width(self, width):
        assert isinstance(width, int)
        self.width = widgets

    def draw(self, frame_counter):
        if frame_counter >= self.intercept_interval:
            return
        _radius = self.max_radius / (self.intercept_interval - frame_counter)
        pygame.draw.circle(
            self.surface, self.color,
            self.w_centrex_y, _radius, width=self.width)


class TargetSurface():
    def __init__(
        self,
        shtbase,
        _manager,
        tkeys,
        tlock,
        dest=None,
        end_pos=None,
        key_lock_ignore_case=True
    ):
        self.shtbase = shtbase
        self.key_lock_ignore_case = key_lock_ignore_case
        self.ps = self.shtbase.ps
        self.FPS = self.ps.FPS
        self.manager = _manager
        self.intercept_chr = \
            self.manager.intercept_keycode and \
            chr(self.manager.intercept_keycode)
        self.moving_speed = self.manager.moving_speed
        self.defense_surface = None
        self.tlock = tlock
        self.tkeys = \
            [k.lower() for k in tkeys] \
            if self.key_lock_ignore_case else \
            tkeys
        self.font_color = (200, 22, 98)
        self.font = self.manager.font
        self.circle_color = (100, 20, 25, 20)
        self.circle_width = 4
        self.intercepted = False
        self.intercept_frame_counter = 0
        self.laser_color = self.manager.laser_color
        self.laser_width = self.manager.laser_width
        self.surface = self.get_surface()
        self.size = self.get_size()
        self.start_pos = self.dest = dest if dest else self.get_default_pos()
        self.end_pos = end_pos if end_pos else self. get_default_end_pos()
        self.moving_counter = 0
        self.center = self.get_center()
        self.bg_color = (20, 10, 200, 100)

    def set_dest(self, dest):
        self.dest = dest

    def set_start_pos(self, pos, reset_dest=False):
        self.start_pos = pos
        if reset_dest:
            self.dest = self.start_pos

    def set_end_pos(self, pos):
        self.end_pos = pos

    def get_default_pos(self):
        return self.get_random_dest_top()

    def get_default_end_pos(self):
        return (None, self.shtbase.w_height - self.shtbase.defense_surface.h)

    def set_circle_color(self, color):
        self.circle_color = color

    def set_circle_width(self, width):
        assert isinstance(width, int)
        self.circle_width = width

    def arrived(self):
        if self.end_pos[0]:
            return self.get_x() + self.get_w() >= self.end_pos[0]
        if self.end_pos[1]:
            return self.get_y() + self.get_h() >= self.end_pos[1]

    def get_surface(self):
        return self.font.render(self.tlock, False, self.font_color)

    def get_x(self):
        return self.dest[0]

    def get_y(self):
        return self.dest[1]

    def get_w(self):
        return self.size[0]

    def get_h(self):
        return self.size[1]

    def get_default_bg_points(self, tip_height=55):
        return \
            [
                (self.get_x(), self.get_y()),
                (self.get_x() + self.get_w(), self.get_y()),
                (self.get_x() + self.get_w(), self.get_y() + self.get_h()),
                (
                    self.get_x() + self.get_w() / 2,
                    self.get_y() + self.get_h() + tip_height
                ),
                (self.get_x(), self.get_y() + self.get_h())
            ]

    def draw_bg(self):
        pygame.draw.polygon(
            self.ps.surface,
            self.bg_color,
            self.get_default_bg_points())

    def calc_dest(self, _add):
        return

    def move(
        self, 
        _add=(1, 1), 
        rel=True, 
        use_func=False):
        
        if use_func:
            self.dest = self.calc_dest(_add)
        elif rel:
            self.dest[0] += _add[0]
            self.dest[1] += _add[1]
        else:
            self.set_dest(_add)
        self.center = self.get_center()
        self.draw_bg()

    def set_laser_color(self, laser_color):
        self.laser_color = laser_color

    def get_laser_color():
        return self.laser_color

    def draw_laser_line(self):
        if self.defense_surface is None:
            self.defense_surface = self.shtbase.defense_surface
        assert self.defense_surface is not None
        pygame.draw.line(
            self.ps.surface, self.laser_color,
            self.defense_surface.center, self.center,
            self.laser_width)

    def get_center(self):
        return [
            self.get_x() + self.get_w() / 2,
            self.get_y() + self.get_h() / 2
        ]

    def get_circle_radius(self):
        return self.get_w() / 2

    def circle(self):
        pygame.draw.circle(
            self.shtbase.surface, self.circle_color,
            self.center, self.get_circle_radius(),
            width=self.circle_width)

    def intercept(self, _result, ignore_case=True):
        if len(_result) < 1:
            return self.intercepted

        if self.intercept_chr is None or \
                _result.endswith(self.intercept_chr):

            if self.intercept_chr:
                _result = _result.replace(self.intercept_chr, '').strip()
            else:
                _result = _result.strip()

            self.intercepted = _result.lower() in self.tkeys

        return self.intercepted

    def get_size(self):
        return self.surface.get_size()

    def set_random_dest_top(self):
        self.dest = self.get_random_dest_top()

    def get_random_dest_top(self):
        return [random.randint(0, self.shtbase.w_width - self.get_w()), 0]

    def copy(self):
        _new = copy.copy(self)
        _new.surface = self.surface.copy()
        _new.set_random_dest_top()
        return _new


class TargetsManager():
    def __init__(
            self,
            shtbase,
            frame_counter=None,
            target_surface_lang_code=None,
            intercept_keycode=None):

        self.shtbase = shtbase
        self.ps = self.shtbase.ps
        self.FPS = self.ps.FPS
        self.moving_surfaces = []
        self.frame_counter = frame_counter or 30
        self.difficulty_index_p1 = self.shtbase.difficulty_index + 1
        self.interval = 1.8 * self.shtbase.FPS
        self.intercept_interval = 0.3 * self.shtbase.FPS
        self.moving_speed = 1
        self.intercepted_color = (175, 10, 175, 100)
        self.laser_color = (0, 0, 255, 90)
        self.laser_width = 2
        self.font_size = 50
        self.target_surface_lang_code = \
            target_surface_lang_code or \
            sys_lang_code
        self.font_path = get_font_path(self.target_surface_lang_code)
        self.font = pygame.font.Font(self.font_path, self.font_size)
        self.surfaces = []
        self.intercept_keycode = intercept_keycode
        self.target_count = 20

    def set_intercept_keycode(self, Pskeycode):
        self.intercept_keycode = keycode

    def get_targets(self, d: int = 0, count=20):
        '''
        return [(key,key,key,key,...,lock)]
        '''
        raise NotImplementedError()

    def save(self, _copy):
        _copy['0x0'] = [
            (s.tkeys, s.tlock, s.dest) for s in self.surfaces]
        _copy['0x1'] = [
            (ms.tkeys, ms.tlock, ms.dest) for ms in self.moving_surfaces]
        return _copy

    def load(self, _copy):
        for keys, lock, dest in _copy['0x0']:
            self.moving_surfaces.append(
                TargetSurface(self.shtbase, self, keys, lock))
        for keys, lock, dest in _copy['0x1']:
            self.moving_surfaces.append(
                TargetSurface(self.shtbase, self, keys, lock, dest))

    def set_target_surface_lang_code(self, lang_code):
        self.target_surface_lang_code = lang_code

    def set_target_count(self, target_count):
        self.target_count = target_count
        self.shtbase.set_target_count(target_count)

    def moving_surfaces_blit(self):
        pass

    def set_font_size(self, size):
        assert isinstance(size, int)
        self.font_size = size

    def get_font_size(self):
        return self.font_size

    def get_target_surfaces(self):
        _targets = self.get_targets(self.shtbase.difficulty_index)
        self.set_target_count(len(_targets))

        return [
            TargetSurface(self.shtbase, self, t[0:-1], t[-1])
            for t in _targets
        ]

    def set_surfaces(self):
        self.surfaces = self.get_target_surfaces()
        self.set_target_count(len(self.surfaces))

    def get_surfaces(self):
        if not self.surfaces:
            self.set_surfaces()
        return self.surfaces

    def count(self):
        return len(self.surfaces)

    def get_random_surface(self):
        random_ws = self.surfaces[
            random.randint(0, self.count - 1)]
        return random_ws.copy()

    def pop_surface(self):
        return self.surfaces.pop()

    def add_moving_surfaces(self):
        ws = self.pop_surface()
        self.moving_surfaces.append(ws)
        self.frame_counter = 0

    def blit_intercepting(self, moving_surfaces):
        pass

    def blit(self):
        if len(self.surfaces) > 0:
            if len(self.moving_surfaces) < 1:
                self.add_moving_surfaces()
            if self.frame_counter >= self.interval:
                self.add_moving_surfaces()

        for w in self.moving_surfaces:
            if w.intercepted:
                if w.intercept_frame_counter >= self.intercept_interval:
                    self.moving_surfaces.remove(w)
                self.blit_intercepting(w)
                w.surface = w.font.render(
                    w.tlock, False,
                    self.intercepted_color)
                self.shtbase.surface.blit(w.surface, w.dest)
                w.circle()
                w.draw_laser_line()
                w.intercept_frame_counter += 1
                continue

            if w.intercept(self.shtbase._input):
                self.shtbase._input = ''
                self.shtbase.input_surface._update()
                self.shtbase.surface.blit(w.surface, w.dest)
                self.shtbase.win_count += 1
                continue

            if w.arrived():
                if self.shtbase.defense_surface.flicker() < 1:
                    self.moving_surfaces.remove(w)
                    self.shtbase.lose_count += 1
                    continue

            w.move((0, self.moving_speed))
            self.shtbase.surface.blit(w.surface, w.dest)
            self.moving_surfaces_blit()

        self.frame_counter += 1


class InputSurface():
    def __init__(
            self,
            shtbase,
            font_lang_code=None,
            font_bold=False,
            border_radius=5,
            border_width=1,
            border_color=(200, 20, 30, 60)):

        self.shtbase = shtbase
        self.ps = self.shtbase.ps
        self.font_size = 55
        self.font_lang_code = font_lang_code or \
            self.shtbase.font_lang_code
        self.font = \
            get_sys_font_by_lang_code(self.font_lang_code) \
            if self.font_lang_code else \
            get_default_font(self.font_size)
        self.font_color = (200, 22, 98)
        self.surface = None
        self.frame_counter = 0
        self.x, self.y = self.dest = (0, 0)
        self.w, self.h = self.rect = (0, 0)
        self.border_radius = border_radius
        self.border_width = border_width
        self.border_color = border_color
        self.font_bold = font_bold

        self.font.set_bold(self.font_bold)

    def update_rect(self, rect=None):
        self.w, self.h = self.rect = rect or self.surface.get_size()

    def update_dest(self, dest=None):
        self.x, self.y = self.dest = dest or (
            self.shtbase.w_width_of_2 - self.w / 2,
            self.shtbase.w_height - self.h)

    def _update(self):
        self.surface = self.font.render(
            self.shtbase._input.strip(),
            False, self.font_color)

    def _blit(self):
        pygame.draw.rect(
            self.shtbase.surface,
            color=self.border_color,
            rect=(self.x, self.y, self.w, self.h),
            width=self.border_width,
            border_radius=self.border_radius)

    def blit(self):
        if self.surface is None:
            return

        self.update_rect()
        self.update_dest()
        self._blit()

        self.shtbase.surface.blit(
            self.surface, self.dest)


class DefenseSurface():
    def __init__(self, shtbase, original_color=None):
        self.shtbase = shtbase
        self.ps = self.shtbase.ps
        self.h = self.shtbase.w_height / 20
        self.surface = pygame.Surface((self.shtbase.w_width, self.h))
        self.original_color = original_color or self.get_default_color()
        self.color = self.original_color
        self.emitter_radius = self.h / 2
        self.emitter_color = None

        self.flicker_interval = 1 * self.ps.FPS  # 2s
        self.flicker_counter = 0
        self.flicker_color = [self.color, (250, 0, 0)]
        self.flicker_color_step = (
            (self.flicker_color[1][0] - self.flicker_color[0][0])
            / self.flicker_interval,
            (self.flicker_color[1][1] - self.flicker_color[0][1])
            / self.flicker_interval,
            (self.flicker_color[1][2] - self.flicker_color[0][2])
            / self.flicker_interval,
        )

        self.center = self.get_center()

    def get_default_color(self):
        return (10, 200, 99)

    def flicker(self):
        self.flicker_counter += 1
        self.color = (
            min(int(self.flicker_color[0][0]
                    + self.flicker_color_step[0] * self.flicker_counter),
                255),
            min(int(self.flicker_color[0][1]
                    + self.flicker_color_step[1] * self.flicker_counter),
                255),
            min(int(self.flicker_color[0][2]
                    + self.flicker_color_step[2] * self.flicker_counter),
                255)
        )
        if self.flicker_counter >= self.flicker_interval:
            self.flicker_counter = 0
            self.color = self.original_color
        return self.flicker_counter

    def set_emitter_color(self, color=(255, 0, 0, 50)):
        self.emitter_color = color

    def get_emitter_color(self):
        return self.emitter_color

    def get_center(self):
        return \
            [
                self.ps.w_width_of_2,
                self.ps.w_height - self.h / 2
            ]

    def draw_emitter(self):
        self.emitter_color = self.set_emitter_color() \
            if self.shtbase.targets_manager is None \
            else self.shtbase.targets_manager.laser_color
        pygame.draw.circle(
            self.ps.surface, self.emitter_color,
            self.center, self.emitter_radius)

    def blit(self):
        self.surface.fill(self.color)
        self.shtbase.surface.blit(
            self.surface, (0, self.shtbase.w_height - self.h))
        self.draw_emitter()


class InfoSurface():
    def __init__(self, shtbase):
        self.shtbase = shtbase
        self.ps = shtbase.ps
        self.surface = self.ps.surface
        self.game_info_dest = (10, 10)
        self.game_info = self.shtbase.name_t + \
            '/' + self.shtbase.difficulties[self.ps.difficulty_index]
        self.game_info_color = (255, 0, 255, 10)
        self.font_size = 25
        self.font = get_default_font(self.font_size)

        self.score_font_size = 55
        self.score_font = get_default_font(self.score_font_size)

        self.datetime_diff_font_size = 50
        self.datetime_diff_font = get_default_font(
            self.datetime_diff_font_size)
        self.datetime_diff_font_color = ...

        self.font = get_default_font(self.font_size)

        self.game_info_surface = self.font.render(
            self.game_info, False, self.game_info_color)

        self.score = 0
        self._pass = False
        self.ps_info_surface = ...

        self.score_surface = ...
        self.datetime_diff_surface = ...
        self.greeting_surface = ...

        self.end_time = self.ps.end_time = None

    def get_score_font_color(self):
        return (20, 255, 0) if self._pass else (255, 20, 0)

    def get_win_info(self):
        return \
            '|' + _('win: ') + str(self.shtbase.win_count) + \
            '|' + _('lose: ') + str(self.shtbase.lose_count) + \
            '|' + _('remain: ') + str(self.shtbase.targets_manager.count()) + \
            '|' + _('total: ') + str(self.shtbase.target_count) + '|'

    def get_win_info_dest(self):
        _w, _ = self.win_info_surface.get_size()
        return [self.ps.w_width - _w, 0]

    def get_datetime_diff_str(self):
        if self.end_time is None:
            self.end_time = self.ps.end_time = datetime.now()
        diff = self.end_time - self.shtbase.start_time + \
            self.shtbase.last_timedelta
        _h, _rem = divmod(diff.seconds, 3600)
        _min, _sec = divmod(_rem, 60)
        return _('Cost: ') + f'{_h}:{_min}:{_sec}'

    def blit(self):
        self.win_info_surface = self.font.render(
            self.get_win_info(), False, self.game_info_color)

        self.surface.blit(self.game_info_surface, self.game_info_dest)
        self.surface.blit(self.win_info_surface, self.get_win_info_dest())

    def get_score(self):
        self.score = int(
            100 *
            self.shtbase.win_count /
            self.shtbase.target_count)
        return self.score

    def get_score_pass(self):
        self._pass = self.score > 60
        return self._pass

    def get_greeting(self):
        _player_name = self.shtbase.get_player_name()
        return \
        (
            _('Success! Dear %s, you are so awesome!') if self._pass else \
            _('Dear %s, practice makes perfect, keep trying!')
        )%_player_name

    def get_score_str(self):
        return _('Score: ') + str(self.score)

    def get_greeting_dest(self):
        _w, _h = self.greeting_surface.get_size()
        _, _s_h = self.score_surface.get_size()
        return [
            self.ps.w_width_of_2 - _w / 2,
            self.ps.w_height_of_2 - _h - _s_h
        ]

    def get_score_surface_dest(self):
        _w, _h = self.score_surface.get_size()
        return [
            self.ps.w_width_of_2 - _w / 2,
            self.ps.w_height_of_2 - _h
        ]

    def get_datetime_diff_surface_dest(self):
        _w, _h = self.datetime_diff_surface.get_size()
        return [
            self.ps.w_width_of_2 - _w / 2,
            self.ps.w_height_of_2 + _h
        ]

    def score_blit(self):
        self.score = self.get_score()
        self.get_score_pass()

        self.greeting_surface = self.score_font.render(
            self.get_greeting(),
            False,
            self.get_score_font_color()
        )

        self.score_surface = self.score_font.render(
            self.get_score_str(),
            False,
            self.get_score_font_color())

        self.datetime_diff_surface = self.datetime_diff_font.render(
            self.get_datetime_diff_str(),
            False,
            self.get_score_font_color())

        self.surface.blit(
            self.greeting_surface,
            self.get_greeting_dest())

        self.surface.blit(
            self.score_surface,
            self.get_score_surface_dest())

        self.surface.blit(
            self.datetime_diff_surface,
            self.get_datetime_diff_surface_dest())


class ShootingBase(GameBase, PsKeyCode):
    def __init__(self, ps, font_lang_code=None):
        assert hasattr(self, 'name_t')
        assert hasattr(self, 'difficulties')
        assert hasattr(self, 'module_str')

        self.ps = ps
        # window
        self.w_width = self.ps.w_width
        self.w_height = self.ps.w_height
        self.w_height_of_2 = self.ps.w_height_of_2
        self.w_width_of_2 = self.ps.w_width_of_2
        self.w_centrex_y = self.ps.w_centrex_y
        self.running = True
        self.FPS = self.ps.FPS
        self.clock = self.ps.clock
        self.player_name = self.ps.player_name
        self._load = False

        self.font_lang_code = font_lang_code or sys_lang_code

        self.subject = self.ps.subject
        self.subject_index = self.ps.subject_index
        self.subject_game_index = self.ps.subject_game_index
        self.difficulty_index = self.ps.difficulty_index

        self.main_menu = self.ps.main_menu
        self.play_menu = self.ps.play_menu
        self.save_menu = self.ps.save_menu
        self.surface = self.ps.surface

        self._input = ''
        self.font = get_sys_font_by_lang_code(self.font_lang_code, 45)
        self.info_surface = self.get_info_surface()
        self.defense_surface = self.get_defense_surface()
        self.input_surface = self.get_input_surface()
        self.copy_path = get_copy_path(self.module_str)

        self.last_timedelta = timedelta(0)
        self.start_time = datetime.now()
        self.end_time = None

        self.targets_manager = self.get_targets_manager()
        self.win_count = 0
        self.lose_count = 0
        self.target_count = 0
        self.print_game_info()

    def get_player_name(self):
        return self.player_name
    
    def set_player_name(self,name):
        self.player_name = self.ps.player_name = name

    def set_input(self, content=''):
        self._input = content

    def set_font_lang_code(self, font_lang_code):
        self.font_lang_code = font_lang_code
        self.font = get_sys_font_by_lang_code(self.font_lang_code, 45)

    def get_info_surface(self):
        return InfoSurface(self)

    def get_input_surface(self):
        return InputSurface(self)

    def get_defense_surface(self):
        return DefenseSurface(self)

    def get_targets_manager(self):
        raise NotImplementedError()

    def set_target_count(self, count):
        self.target_count = count

    def print_game_info(self):
        print(self.subject.name_t, self.name_t,
              self.difficulties[self.difficulty_index])

    def key_clean(self, code):
        return 48 <= code <= 57 or code == 45

    def handle_events(self, events):
        for e in events:
            if e.type == pygame.QUIT:
                exit()
            elif e.type == pygame.KEYDOWN:
                if e.key == pygame.K_ESCAPE:
                    self.save_menu._menu.enable()
                    self.last_timedelta += datetime.now() - self.start_time
                    self.save_menu._menu.mainloop(self.surface)
                    self.start_time = datetime.now()
                    return
                elif e.key == pygame.K_BACKSPACE:
                    self._input = self._input[0:-1]
                    self.input_surface._update()
                    return
                elif self.key_clean(e.key):
                    self._input += chr(e.key)
                    self.input_surface._update()
                    return

    def load(self):
        try:
            self._load = True
            with open(self.copy_path, 'rb') as f:
                _copy = pickle.load(f)
            self.targets_manager.load(_copy)
            self.target_count, self.win_count, self.lose_count = _copy['0x2']
            self.last_timedelta = _copy['0x3']
            self.start()
        except e:
            print(e)

    def save(self):
        _copy = {}
        self.targets_manager.save(_copy)
        _copy['0x2'] = (self.target_count, self.win_count, self.lose_count)
        _copy['0x3'] = (datetime.now() - self.start_time) + self.last_timedelta

        # https://docs.python.org/3/library/pickle.html?highlight=pickle
        # Warning:
        # The pickle module is not secure. Only unpickle data you trust.
        with open(self.copy_path, 'wb') as f:
            pickle.dump(_copy, f)

    def _start(self):
        if not self._load:
            self.targets_manager.set_surfaces()

    def play(self):
        self._load = False
        self.targets_manager.surfaces = []
        self.targets_manager.moving_surfaces = []
        self.targets_manager.set_surfaces()
        self.start()

    def all_targets_gone(self):
        return self.win_count + self.lose_count < self.target_count

    def start(self):

        self._start()

        while self.running:
            self.clock.tick(self.FPS)

            self.surface.fill((0, 0, 0))

            events = pygame.event.get()
            self.handle_events(events)
            if self.main_menu._menu.is_enabled():
                self.main_menu._menu.update(events)

            if self.play_menu._menu.is_enabled():
                self.play_menu._menu.update(events)

            if self.all_targets_gone():
                self.info_surface.blit()
                self.defense_surface.blit()
                self.targets_manager.blit()
                self.input_surface.blit()
            else:
                self.info_surface.score_blit()

            pygame.display.update()




import gettext
import locale
import os
import sys

sys_lang_code = locale.getdefaultlocale()[0]
locale_path = locale_dir_path = os.path.abspath(os.path.dirname(__file__))
locale_langcodes =  \
    [d for d in os.listdir(locale_path)
     if os.path.isdir(os.path.join(locale_path, d))]

if sys_lang_code not in locale_langcodes:
    sys_lang_code = 'en_US'

lang = gettext.translation(
    'primaryschool', localedir=locale_path,
    languages=[sys_lang_code])

lang.install()

_ = lang.gettext

t = T = _




import os
import random
import sys
from typing import Any, List, Optional, Sequence, Text, Tuple, Union, overload

import pygame
import pygame_menu
from pygame.locals import *
from pygame_menu._widgetmanager import WidgetManager
from pygame_menu.widgets import *
from xpinyin import Pinyin

from primaryschool.locale import _
from primaryschool.resource import *
from primaryschool.subjects import *

name_t = _('Yuwen')




import importlib
import os
import pickle
import threading
from functools import partial
from itertools import zip_longest

import pygame
import pygame_menu
from pygame.locals import *
from pygame_menu._scrollarea import ScrollArea
from pygame_menu.baseimage import BaseImage
from pygame_menu.locals import *
from pygame_menu.widgets import *

from primaryschool.dirs import *
from primaryschool.locale import _
from primaryschool.resource import (default_font, default_font_path,
                                    get_default_font, get_resource_path)
from primaryschool.settings import *
from primaryschool.subjects import subjects

app_description_t = _("app_description_t")


class SaveMenu():
    def __init__(
            self,
            ps):
        self.ps = ps
        self.surface = self.ps.surface
        self.title = _('Save game?')
        self._menu = self.ps.get_default_menu(self.title)
        self.save = False

    def add_widgets(self):

        self._menu.add.button(
            _('Save and return'),
            self.save_the_game,
            font_name=self.ps.font_path)
        self._menu.add.button(
            _('Continue'),
            self.continue_the_game,
            font_name=self.ps.font_path)
        self._menu.add.button(
            _('Return to main menu'),
            self.to_main_menu,
            font_name=self.ps.font_path)

    def to_main_menu(self):
        self.ps.main_menu._menu.full_reset()
        self.ps.main_menu._menu.enable()
        self.ps.main_menu._menu.mainloop(self.surface)

    def save_the_game(self):
        self.ps.subject_game.save(self.ps)
        self._menu.disable()
        self.to_main_menu()

    def continue_the_game(self):
        self._menu.disable()


class ContributorsMenu():
    def __init__(
            self,
            ps):
        self.ps = ps
        self.title = _('Contributors & Sponsors')
        self._menu = self.ps.get_default_menu(
            self.title, onclose=pygame_menu.events.EXIT)
        self._font = get_default_font(15)
        self._head_font = get_default_font(18)
        self.return_background_color = (200, 200, 255, 200)

    def add_widgets(self):
        contributors_table = self._menu.add.table()
        contributors_table.default_cell_padding = 5
        contributors_table.default_row_background_color = 'white'
        contributors_table.add_row(
            ['Contributors', 'name', ' ', 'Sponsors', 'name'],
            cell_font=self._head_font)

        _len = list(range(max(len(app_contributors), len(app_sponsors))))
        for l, c, s in zip_longest(_len, app_contributors, app_sponsors):
            c, s = c or ' ', s or ' '
            contributors_table.add_row(
                [l + 1, c, ' ', l + 1, s],
                cell_font=self._font)

        self._menu.add.button(
            _('Return to main menu'),
            pygame_menu.events.BACK,
            font_name=self.ps.font_path,
            background_color=self.return_background_color,
            float=True, align=pygame_menu.locals.ALIGN_RIGHT)


class AboutMenu():
    def __init__(
            self,
            ps):

        self.ps = ps
        self.title = _('About')
        self._menu = self.ps.get_default_menu(self.title)
        self.app_name_font = get_default_font(50)
        self.app_version_font = get_default_font(20)
        self.app_description_font = get_default_font(22)
        self.app_url_font = get_default_font(20)
        self.app_author_font = self.app_contributors_font = \
            get_default_font(20)
        self.contributors_menu = ContributorsMenu(self.ps)
        self._label_font = get_default_font(32)

    def add_widgets(self):
        self._menu.add.label(
            app_name, max_char=-1,
            font_name=self.app_name_font)
        self._menu.add.label(
            app_version, max_char=-1,
            font_name=self.app_version_font)
        self._menu.add.label(
            app_description_t, max_char=-1,
            font_name=self.app_description_font)
        self._menu.add.url(
            app_url, font_name=self.app_url_font)
        self._menu.add.label(
            _('Author'), max_char=-1,
            font_name=self._label_font)
        self._menu.add.label(
            app_author, max_char=-1,
            font_name=self.app_author_font)

        self.contributors_menu.add_widgets()

        self._menu.add.button(
            _('Contributors & Sponsors'),
            self.contributors_menu._menu,
            font_name=self._label_font)

        self._menu.add.button(
            _('Return to main menu'),
            pygame_menu.events.BACK,
            font_name=self.ps.font_path)


class PlayMenu():
    def __init__(
            self,
            ps):
        self.ps = ps
        self.title = _('Play Game')
        self.player_name = self.ps.player_name
        self.player_name_text_input = None
        self._menu = self.ps.get_default_menu(self.title)
        self.subjects = self.ps.subjects
        self.subject_games = self.ps.subject_games
        self.subject_index = self.ps.subject_index = 0
        self.subject_game_index = self.ps.subject_game_index
        self.difficulty_index = self.ps.difficulty_index
        self.subject = self.ps.subject
        self.subject_game = self.ps.subject_game
        self.subject_dropselect = None
        self.subject_game_dropselect = None
        self.difficulty_dropselect = None
        self.continue_button = None
        self.selection_box_bgcolor = (255, 255, 255)
        self.help_lael = None
        self.help_lael_font = get_default_font(20)
        self.help_lael_bg = (228, 0, 252, 30)
        self.help_lael_border_color = (228, 0, 252, 200)

    def set_player_name(self, name):
        self.player_name = self.ps.player_name = name
        if self.player_name_text_input:
            self.player_name_text_input.set_value(
                self.player_name)
    
    def get_player_name(self):
        return self.player_name

    def add_widgets(self):
        self.player_name_text_input = self._menu.add.text_input(
            title=_('Name :'),
            default=self.get_player_name(),
            font_name=self.ps.font_path)

        self.subject_dropselect = self._menu.add.dropselect(
            title=_('Subject :'),
            items=[(s.name_t, index) for index, s in
                   enumerate(self.subjects)],
            font_name=self.ps.font_path,
            default=0,
            selection_box_bgcolor=self.selection_box_bgcolor,
            placeholder=_('Select a Subject'),
            onchange=self.on_subject_dropselect_change
        )
        self.subject_game_dropselect = self._menu.add.dropselect(
            title=_('Game :'),
            items=[(g.name_t, index) for index, g in
                   enumerate(self.subject_games)],
            font_name=self.ps.font_path,
            default=0,
            selection_box_bgcolor=self.selection_box_bgcolor,
            placeholder=_('Select a game'),
            onchange=self.on_subject_game_dropselect_change
        )

        self.difficulty_dropselect = self._menu.add.dropselect(
            title=_('Difficulty :'),
            items=[(d, index) for index, d in
                   enumerate(self.subject_games[0].difficulties)],
            font_name=self.ps.font_path,
            default=0,
            selection_box_bgcolor=self.selection_box_bgcolor,
            placeholder=_('Select a difficulty'),
            onchange=self.on_difficulty_dropselect_change
        )
        self.update_selection_box_width()
        self.set_difficulty_index()

        self._menu.add.button(
            _('Play'),
            self.play_btn_onreturn,
            font_name=self.ps.font_path)

        self.continue_button = self._menu.add.button(
            _('Continue'),
            self.continue_btn_onreturn,
            font_name=self.ps.font_path)
        self.update_continue_button()

        self._menu.add.button(
            _('Return to main menu'),
            pygame_menu.events.BACK,
            font_name=self.ps.font_path)

        self.help_lael = self._menu.add.label(
            '', font_name=self.help_lael_font)
        self.update_help_lael()

    def update_selection_box_width(self):
        for ds in [
                self.subject_dropselect,
                self.subject_game_dropselect,
                self.difficulty_dropselect]:

            ds._selection_box_width = max(
                [b.get_width() for b in ds._option_buttons]
            ) + ds._selection_box_inflate[0]
            ds._make_selection_drop()
            ds.render()

    def play_btn_onreturn(self):
        self.start_the_game()

    def continue_btn_onreturn(self):
        self.start_copied_game()

    def update_continue_button(self):
        if self.subject_game.has_copy():
            self.continue_button.show()
        else:
            self.continue_button.hide()

    def update_subject_game_dropselect(self):
        self.subject_game_dropselect.update_items(
            [(g.name_t, index) for index, g in enumerate(
                self.subject.games)])
        self.subject_game_dropselect.set_value(self.subject_game_index)

    def update_difficulty_dropselect(self):
        self.difficulty_dropselect.update_items(
            [(d, index) for index, d in enumerate(
                self.subject_game.difficulties)])
        self.difficulty_dropselect.set_value(self.difficulty_index)

    def update_help_lael(self):
        self.help_lael.set_title(self.subject_game.help_t.strip())
        self.help_lael.set_background_color(self.help_lael_bg)
        self.help_lael.set_border(2, self.help_lael_border_color)

    def start_copied_game(self):
        self.subject_game.load(self.ps)

    def start_the_game(self):
        self.subject_game.play(self.ps)

    def on_difficulty_dropselect_change(self, value, index):
        self.set_difficulty_index(index)

    def on_subject_dropselect_change(self, item, index):
        self.set_subject_index(index)

    def on_subject_game_dropselect_change(self, item, index):
        self.set_subject_game_index(index)

    def set_subject_index(self, index=0):
        self.subject_index = self.ps.subject_index = index
        self.subject = self.ps.subject = self.subjects[self.subject_index]
        self.subject_games = self.ps.subject_games = self.subject.games
        self.set_subject_game_index()
        self.update_subject_game_dropselect()
        self.update_selection_box_width()

    def set_subject_game_index(self, index=0):
        self.subject_game_index = self.ps.subject_game_index = index
        self.subject_game = self.ps.subject_game = \
            self.subject.games[self.subject_game_index]
        self.update_continue_button()
        self.set_difficulty_index()
        self.update_help_lael()

    def set_difficulty_index(self, index=0):
        self.difficulty_index = \
            self.ps.difficulty_index = index if index != 0 else \
            self.subject_game.default_difficulty_index
        self.update_difficulty_dropselect()


class MainMenu():
    def __init__(self, ps):
        self.ps = ps
        self.title = _('Primary School')
        self._menu = self.ps.get_default_menu(self.title)
        self.play_menu = self.ps.play_menu
        self.about_menu = self.ps.about_menu

    def add_widgets(self):
        self._menu.add.button(
            _('Play'), self.ps.play_menu._menu,
            font_name=self.ps.font_path,)
        self._menu.add.button(
            _('About'), self.ps.about_menu._menu,
            font_name=self.ps.font_path,)
        self._menu.add.button(
            _('Quit'), pygame_menu.events.EXIT,
            font_name=self.ps.font_path,)


class PrimarySchool():
    def __init__(self):
        pygame.init()
        self.running = True
        self.surface = pygame.display.set_mode(
            (0, 0), pygame.RESIZABLE)
        self.w_width, self.w_height = self.surface.get_size()
        self.w_width_of_2, self.w_height_of_2 = \
            self.w_width / 2, self.w_height / 2
        self.w_centrex_y = [self.w_width_of_2, self.w_height]
        self.FPS = 30
        self.player_name = _('default_name')
        self.clock = pygame.time.Clock()
        self.subjects = subjects
        self.subject_games = self.subjects[0].games
        self.subject_index = 0
        self.subject_game_index = 0
        self.difficulty_index = 0
        self.subject = self.subjects[0]
        self.subject_game = self.subject_games[0]
        self.font_path = default_font_path
        self.font = default_font
        self.bg_img = None
        self.play_menu = PlayMenu(self)
        self.about_menu = AboutMenu(self)
        self.save_menu = SaveMenu(self)
        self.main_menu = MainMenu(self)

    def add_widgets(self):
        self.play_menu.add_widgets()
        self.about_menu.add_widgets()
        self.save_menu.add_widgets()
        self.main_menu.add_widgets()

    def set_bg_img(self, src_name='0x1.png'):
        self.bg_img = BaseImage(
            get_resource_path(src_name),
            pygame_menu.baseimage.IMAGE_MODE_FILL)

    def get_bg_img(self):
        if not self.bg_img:
            self.set_bg_img()
        return self.bg_img

    def get_default_menu(self, title, **kwargs):
        theme = pygame_menu.themes.THEME_BLUE.copy()
        theme.title_font = theme.widget_font = self.font
        theme.title_bar_style = pygame_menu.widgets.MENUBAR_STYLE_NONE
        theme.background_color = self.get_bg_img()
        return pygame_menu.Menu(
            title,
            self.w_width, self.w_height,
            theme=theme, **kwargs)

    def clear_screen(self):
        self.surface.fill((255, 255, 255))
        pygame.display.update()

    def run(self):

        self.add_widgets()

        while self.running:
            self.clock.tick(self.FPS)
            events = pygame.event.get()
            for event in events:
                if event.type == pygame.QUIT:
                    exit()
            if self.main_menu._menu.is_enabled():
                self.main_menu._menu.mainloop(self.surface)

            pygame.display.flip()


def go():
    PrimarySchool().run()
    pass



import copy
import os
import pickle
import random
import sys
from datetime import datetime, timedelta
from typing import Any, List, Optional, Sequence, Text, Tuple, Union, overload

import pygame
import pygame_menu
from pygame.locals import *
from xpinyin import Pinyin

from primaryschool.dirs import *
from primaryschool.locale import _
from primaryschool.resource import (default_font, default_font_path,
                                    get_default_font, get_font_path,
                                    get_resource_path)
from primaryschool.subjects import *
from primaryschool.subjects._abc_ import GameBase
from primaryschool.subjects._common_.shootingbase import *
from primaryschool.subjects.english.g_chinese_english_missile.words import \
    cn_ps_e_words

# primaryschool.subjects.yuwen.g_pinyin_missile
module_str = __name__

name_t = _('Chinese-English Missile')

difficulties = [
    _('Grade 3.1'),  # 0
    _('Grade 3.2'),  # 1
    _('Grade 4.1'),  # 2
    _('Grade 4.2'),  # 3
    _('Grade 5.1'),  # 4
    _('Grade 5.2'),  # 5
    _('Grade 6.1'),  # 6
    _('Grade 6.2'),  # 7
    _('All grades'),  # 8
]


help_t = _('''
Enter the corresponding words of the Chinese characters.
''')


class ZhCNEmTargetsManager(TargetsManager):
    def __init__(self, shtbase):
        super().__init__(
            shtbase,
            target_surface_lang_code='zh_CN'
        )
        self.wave = ShootingWave(self)

    def blit_intercepting(self, moving_surfaces):
        self.wave.draw(moving_surfaces.intercept_frame_counter)

    def get_cn_ps_e_words(self, g_index: int):
        words = []
        if g_index < 8:
            words = cn_ps_e_words[g_index]
        elif g_index == 8:
            words = sum(cn_ps_e_words, [])
        return sum(words, [])

    def get_targets(self, d: int = 0, count=30):
        return self.get_cn_ps_e_words(d)


class ZhCNEnglishMissile(ShootingBase):
    def __init__(self, ps):
        self.name_t = name_t
        self.difficulties = difficulties
        self.module_str = module_str
        super().__init__(ps)

    def get_targets_manager(self):
        return ZhCNEmTargetsManager(self)

    def key_clean(self, code):
        return \
            self.keycode_in_pure_num(code) or \
            self.keycode_in_alpha(code) or \
            self.keycode_in_space(code) or \
            self.keycode_in_hyphen(code)


def enjoy(ps):
    return ZhCNEnglishMissile(ps)




class PsKeyCode():
    def __init__(self):
        pass

    def keycode_in_alpha_upper(self, code):
        return 65 <= code <= 90

    def keycode_in_alpha_lower(self, code):
        return 97 <= code <= 122

    def keycode_in_alpha(self, code):
        return \
            self.keycode_in_alpha_lower(code) or \
            self.keycode_in_alpha_upper(code)

    def keycode_in_num_neg(self, code):
        return \
            self.keycode_in_pure_num(code) or \
            self.keycode_in_hyphen(code)

    def keycode_in_num_float(self, code):
        return \
            self.keycode_in_pure_num(code) or \
            self.keycode_in_dot(code)

    def keycode_in_pure_num(self, code):
        return 48 <= code <= 57

    def keycode_in_num(self, code):
        return \
            self.keycode_in_pure_num(code) or \
            self.keycode_in_hyphen(code) or \
            self.keycode_in_dot(code)

    def keycode_in_dot(self, code):
        return code == 46

    def keycode_in_alpha_num(self, code):
        return \
            self.keycode_in_num(code) or \
            self.keycode_in_alpha(code)

    def keycode_in_space(self, code):
        return code == 32

    def keycode_in_hyphen(self, code):
        return code == 45

    def keycode_in_return(self, code):
        return code == 0xd




import os
import sys
from importlib import import_module

from primaryschool.dirs import *
from primaryschool.locale import _

subject_module_prefix = 'primaryschool.subjects.'
subject_dir_path = os.path.abspath(os.path.dirname(__file__))

media_dir_path = os.path.join(subject_dir_path, 'media')
img_dir_path = os.path.join(media_dir_path, 'img')
default_game_image_path = os.path.join(img_dir_path, '0x2.png')
default_subject_image_path = os.path.join(img_dir_path, '0x3.png')


def game_is_valid(subject_name, game_name):
    return \
        game_name.startswith('g_') and \
        os.path.exists(
            os.path.join(
                subject_dir_path, subject_name, game_name, '__init__.py'))


def subject_is_valid(subject_path, subject):
    return \
        os.path.isdir(subject_path) and \
        not subject.startswith('_') and \
        not subject == 'media'


def get_subject_tree():
    subject_names = []
    game_modules = []

    for _subject in os.listdir(subject_dir_path):
        _subject_path = os.path.join(subject_dir_path, _subject)

        if not subject_is_valid(_subject_path, _subject):
            continue

        no_game = True
        for _game in os.listdir(_subject_path):
            if not game_is_valid(_subject, _game):
                continue
            no_game = False
            game_modules.append(
                subject_module_prefix + _subject + '.' + _game)

        # Subject without game is ignored.
        if not no_game:
            subject_names.append(_subject)

    return subject_names, game_modules


subject_names, game_modules = get_subject_tree()


class Game():
    def __init__(
            self,
            module_str,
            subject,
            ps=None):
        self.module_str = module_str
        self.subject = subject
        self.ps = ps
        self.module = import_module(self.module_str)
        self._game = None
        self.name = self.module_str.split('.')[-1]
        self.name_t = getattr(self.module, 'name_t')
        self.image_path = getattr(
            self.module, 'image_path', default_game_image_path)
        self.help_t = getattr(self.module, 'help_t')
        self.difficulties = getattr(self.module, 'difficulties')
        self.default_difficulty_index = getattr(
            self.module,
            'default_difficulty_index', 0)

    def get_game(self, ps):
        if not self._game:
            self._game = self.module.enjoy(ps)
        return self._game

    def play(self, ps):
        ps.play_menu._menu.disable()
        ps.play_menu._menu.full_reset()
        self._game = None
        self.get_game(ps).play()

    def save(self, ps):
        self.get_game(ps).save()

    def load(self, ps):
        ps.play_menu._menu.disable()
        ps.play_menu._menu.full_reset()
        self.get_game(ps).load()

    def has_copy(self):
        return os.path.exists(
            get_copy_path(
                self.module_str))


class Subject():
    def __init__(
            self,
            name):
        self.name = name
        self.module = import_module(subject_module_prefix + self.name)
        self.name_t = self.module.name_t
        self.image_path = getattr(self.module, 'image_path',
                                  default_subject_image_path)
        self.games = []

    def set_games(self):
        for g in game_modules:
            if g.startswith(subject_module_prefix + self.name):
                self.games.append(Game(g, self))

    def get_games(self):
        if len(self.games) < 1:
            self.set_games()
        return self.games

    def get_name_t(self):
        self.name_t = self.module.name_t
        return self.name_t


subjects = [Subject(n) for n in subject_names]


all_games = sum([s.get_games() for s in subjects], [])




import os
import sys
from pathlib import Path

from primaryschool.locale import _
from primaryschool.subjects import *

dir_path = Path(__file__).parent.absolute()
name_t = _('Math')
image_path = os.path.join(dir_path, 'media', 'img', '0x0.png')



# # 人教版小学英语单词表'),
# ('# 三年级上册词汇'),
# ('# 人教版小学英语三年级上册单词表（三会）Unit 1'),
cn_ps_e_words = \
    [
        # 3.1
        [
            [
                # Unit 1
                ('pen', '钢笔'), ('pencil', '铅笔'), ('pencil-case', '铅笔盒'),
                ('ruler', '尺子'), ('eraser', '橡皮'), ('crayon', '蜡笔'),
                ('book', '书'), ('bag', '书包'), ('sharpener', '卷笔刀'),
                ('school', '学校'),
            ],
            [
                # Unit 2
                ('head', '头'), ('face', '脸'), ('nose', '鼻子'),
                ('mouth', '嘴'), ('eye', '眼睛'), ('ear', '耳朵'),
                ('arm', '胳膊'), ('finger', '手指'), ('leg', '腿'),
                ('foot', '脚'), ('body', '身体'),
            ],
            [
                # Unit 3
                ('red', '红色'), ('yellow', '黄色'), ('green', '绿色'),
                ('blue', '蓝色'), ('purple', '紫色'), ('white', '白色'),
                ('black', '黑色'), ('orange', '橙色'), ('pink', '粉色'),
                ('brown', '棕色'),
            ],
            [
                # Unit 4
                ('cat', '猫'), ('dog', '狗'), ('monkey', '猴子'),
                ('panda', '熊猫'),
                ('rabbit', '兔子'), ('duck', '鸭子'), ('pig', '猪'),
                ('bird', '鸟'),
                ('bear', '熊'), ('elephant', '大象'), ('mouse', '老鼠'),
                ('squirrel', '松鼠'),
            ],
            [
                # Unit 5
                ('cake', '蛋糕'), ('bread', '面包'), ('hot dog', '热狗'),
                ('hamburger', '汉堡包'), ('chicken', '鸡肉'),
                ('French fries', '榨薯条'),
                ('Coke', '可乐'), ('juice', '果汁'), ('milk', '牛奶'),
                ('water', '水'),
                ('tea', '茶'), ('coffee', '咖啡'),
            ],
            [
                # Unit 6
                ('one', '一'), ('two', '二'), ('three', '三'), ('four', '四'),
                ('five', '五'), ('six', '六'), ('seven', '七'), ('eight', '八'),
                ('nine', '九'), ('ten', '十'), ('doll', '玩具娃娃'),
                ('boat', '小船'),
                ('ball', '球'), ('kite', '风筝'), ('balloon', '气球'),
                ('car', '小汽车'),
                ('plane', '飞机'),
            ],
        ],
        # 3.2
        [
            [
                # Unit 1
                ('boy', '男孩'), ('girl', '女孩'), ('teacher', '教师'),
                ('student', '学生'), ('this', '这个'), ('my', '我的'),
                ('friend', '朋友'), ('I\'m', '我是'), ('I am', '我是'),
                ('nice', '好的,愉快的'), ('good morning', '早上好'),
                ('good afternoon', '下午好'), ('meet', '遇见；碰见'),
                ('goodbye', '再见'), ('too', '也；太'),
            ],
            [
                # Unit 2
                ('father', '爸爸'), ('dad', '爸爸(口语)'),
                ('mother', '母亲；妈妈'),
                ('mom', '妈妈口语'), ('man', '男人'), ('woman', '女人'),
                ('grandmother', '外祖母'), ('grandma', '[口语]外祖母'),
                ('grandfather', '(外)祖父'), ('grandpa', '[口语](外)祖父'),
                ('sister', '姐妹'), ('brother', '兄妹'), ('let\'s', '让我们'),
                ('let us', '让我们'), ('great', '太好了'),
                ('really', '真地；确切地'),
                ('and', '与；并且'),
            ],
            [
                # Unit 3
                ('eleven', '十一'), ('twelve', '十二'), ('thirteen', '十三'),
                ('fourteen', '十四'), ('fifteen', '十五'), ('sixteen', '十六'),
                ('seventeen', '十七 eighteen', '十八'), ('nineteen', '十九'),
                ('twenty', '二十'), ('how many', '多少'), ('can', '能够；可以'),
                ('look at', '瞧；瞧'),
            ],
            [
                # Unit 4'),
                ('peach', '桃'), ('pear', '梨'), ('orange', '橙子'),
                ('watermelon', '西瓜'), ('apple', '苹果'), ('banana', '香蕉'),
                ('strawberry', '草莓'), ('grape', '葡萄'), ('like', '喜欢'),
                ('some', '一些；某些'), ('thanks', '多谢'),
            ],
            [
                # Unit 5
                ('bus', '公共汽车'), ('bike', '自行车'), ('taxi', '出租车'),
                ('jeep', '吉普车'), ('desk', '课桌'), ('chair', '椅子'),
                ('Walkman', '随身听'), ('lamp', '台灯'), ('your', '您的；您们的'),
                ('zoo', '动物园'),
            ],
            [
                # Unit 6
                ('small', '小的'), ('big', '大的'), ('long', '长的'),
                ('short', '短的；矮的'), ('tall', '高的'), ('giraffe', '长颈鹿'),
                ('deer', '鹿'),
            ],
        ],
        # 4.1
        # ('四年级上册词汇'),
        [
            [
                # ('人教版小学英语四年级上册单词表（三会）Unit 1'),
                ('window', '窗户'), ('have', '（我们）有'), ('board', '写字板'),
                ('new', '新的'), ('light', '灯，灯管'), ('go', '去'),
                ('picture', '画，图画'), ('where', '在哪里'), ('door', '门'),
                ('computer', '计算机'), ('floor', '地板'),
                ('teacher\'s desk', '讲台'),
                ('classroom', '教室'), ('wall', '墙'), ('many', '许多的'),
                ('fan', '扇子，电扇'), ('our', '我们的'),
                ('clean', '打扫，清洁，擦干净的'),
                ('classmate', '同学'), ('good idea', '好主意'),
                ('have a look', '瞧一瞧'), ('all right', '好吧，好的'),
                ('seat', '座位'), ('good job', '干的好'), ('near', '在……的旁边'),
                ('you', '您'), ('what', '什么'), ('see', '瞧'),
                ('in', '在……里面'),
                ('me', '我'), ('the', '这个，这里'), ('look at', '瞧……'),
                ('we', '我们'),
            ],
            [
                # ('人教版小学英语四年级上册单词表（三会）Unit 2'),
                ('Chinese book', '语文书'), ('pencil', '铅笔'),
                ('English book', '英语书'),
                ('twenty-one', '二十一'), ('math book', '数学书'),
                ('thirty-one', '三十一'), ('schoolbag', ''), ('书包'),
                ('forty-one', '[wʌn]'), ('四十一'), ('story-book', '故事书'),
                ('fifty', '五十'), ('notebook', '笔记本'), ('too many', '太多了'),
                ('colour', '颜色'), ('fat', '胖的'), ('heavy', '重的；沉重的'),
                ('may', '可以'), ('what\'s', '什么'), ('what is', '什么'),
                ('sure', '当然可以'), ('sorry', '对不起'),
                ('here you are', '给您'),
            ],
            [
                # ('人教版小学英语四年级上册单词表（三会）Unit 3'),
                ('long hair', '长头发'), ('short hair', '短头发'),
                ('thin', '瘦的'),
                ('strong', '健壮的'), ('quiet', '安静的'),
                ('friend', '朋友'), ('friends', '朋友(复数)'),
                ('Chinese', '中国的'),
                ('like', '喜欢'), ('his', '她的'), ('photo', '照片'),
                ('she has', '她有'), ('he has', '他有'), ('name', '名字'),
                ('he', '她'),
                ('teacher', '教师'), ('student', '学生'), ('music', '音乐'),
                ('science', '科学'), ('sports', '体育运动'),
                ('computer game', '电脑游戏'), ('painting', '绘画'),
                ('her', '她的'), ('you are', '你是'), ('she', '她'),
                ('right', '对的；正确的'), ('boy', '男孩'), ('girl', '女孩'),
                ('friend', '朋友'),
            ],
            [
                # ('人教版小学英语四年级上册单词表（三会）Unit 4'),
                ('study', '书房'), ('bathroom', '卫生间'), ('bedroom', '卧室'),
                ('living room', '客厅'), ('kitchen', '厨房'), ('it is', '它是'),
                ('hey', '嘿'), ('fish', '鱼'), ('is not', 'isn\'t', '不是'),
                ('home', '家'), ('room', '房间'), ('school', '学校'),
                ('classroom', '教室'), ('phone', '电话'), ('bed', '床'),
                ('sofa', '沙发'), ('shelf', '书架'), ('fridge', '冰箱'),
                ('table', '桌子'), ('they', '他们; 她们; 它们'),
                ('aren\'t', 'are not', '不是'),
                ('they\'re', 'they are', '他/她/它们是'),
                ('key', '钥匙'), ('open', '打开'), ('look', '瞧'),
                ('please', '请'),
                ('on', '在……上面'), ('no', '不；不就是'), ('window', '窗户'),
                ('desk', '课桌；书桌'), ('door', '门'), ('chair', '椅子'),
                ('bed', '床'),
            ],
            [
                # ('人教版小学英语四年级上册单词表（三会）Unit 5'),
                ('rice', '米饭'), ('fish', '鱼'), ('noodle', '面条'),
                ('noodles', '面条（复数）'), ('beef', '牛肉'),
                ('vegetable', '蔬菜'),
                ('soup', '汤'), ('have', '吃'), ('dinner', '晚餐；正餐'),
                ('wait', '等'), ('I\'d like', 'I would like', '我想要'),
                ('bread', '面包'), ('milk', '牛奶'), ('egg', '蛋'),
                ('water', '水'), ('hungry', '饥饿的'), ('for', '为；给'),
                ('thank you', '谢谢您'), ('knife', '刀'),
                ('chopsticks', '筷子（复数）'), ('spoon', '勺子'),
                ('plate', '盘子'), ('fork', '叉子'), ('help', '帮助；帮忙'),
                ('pass', '传递'), ('ready', '准备好了'), ('try', '常识；试一下'),
                ('help yourself', '别客气;随意;吃,不用害羞'), ('show', '展示'),
                ('yummy', '好吃的'), ('food', '食物'), ('use', '使用'),
                ('chicken', '鸡肉'),
            ],
            [
                # ('人教版小学英语四年级上册单词表（三会）Unit 6'),
                ('family', '家庭'), ('parents', '父母'), ('uncle', '叔叔；舅舅'),
                ('aunt', '姑姑；婶；姨'), ('baby', '婴儿'), ('people', '人'),
                ('member', '成员'), ('only', '只有；仅仅'), ('puppy', '小狗'),
                ('gee', '用以表示惊奇、赞赏'), ('come', '来'), ('who', '谁'),
                ('sister', '姐妹'), ('brother', '兄弟'),
                ('father', '父亲；爸爸'),
                ('baseball player', '棒球运动员'), ('driver', '司机'),
                ('doctor', '医生'), ('farmer', '农民'), ('nurse', '护士'),
                ('look', '瞧上去；瞧；瞧'), ('young', '年轻的'),
            ],
        ],
        # ('四年级下册词汇'),
        [
            [
                # ('人教版小学英语四年级下册单词表（三会）Unit 1'),
                ('computer', '计算机'), ('board', '写字板'), ('fan', '风扇'),
                ('light', '灯'), ('this', '这；这个'), ('is', '就是'),
                ('my', '我的'), ('that', '那；那个'), ('your', '您的'),
                ('teacher\'s desk', '讲台'), ('picture', '图画；照片'),
                ('wall', '墙壁'),
                ('floor', '地板'), ('yes', '就是；就是的'), ('it', '它'),
            ],
            [
                # ('人教版小学英语四年级下册单词表（三会）Unit 2'),
                ('class', '课程'), ('one', '一'),
                ('two', '二'), ('three', '三'), ('four', '四'),
                ('five', '五'), ('six', '六'), ('seven', '七'),
                ('eight', '八'),
                ('nine', '九'), ('ten', '十'), ('what', '什么'),
                ('time', '时间'),
                ('it\'s', 'it is', '它是'), ('o\'clock', '…点钟'),
                ('math', '数学'),
                ('Chinese', '语文'), ('English', '英语'), ('PE', '体育'),
                ('music', '音乐'), ('for', '为；给'),

            ],
            [
                # ('人教版小学英语四年级下册单词表（三会）Unit 3'),
                ('jacket', '夹克衫'), ('shirt', '衬衫'), ('skirt', '裙子'),
                ('dress', '连衣裙'), ('T-shirt', 'T恤衫'), ('red', '红色的'),
                ('blue', '蓝色的'), ('yellow', '黄色的'), ('green', '绿色的'),
                ('white', '白色的'), ('no', '不；不就是'), ('not', '不；不就是的'),
                ('colour', 'color', '颜色'),

            ],
            [
                # ('人教版小学英语四年级下册单词表（三会）Unit 4'),
                ('warm', '暖与的'), ('cold', '寒冷的'), ('cool', '凉爽的'),
                ('today', '今天'), ('jeans', '牛仔裤'), ('pants', '长裤'),
                ('socks', '袜子'), ('shoes', '鞋子'),
                ('let\'s', 'let us', '让我们;我们一起'),
                ('play', '玩;踢'), ('football', '足球'), ('snowy', '下雪的'),
                ('sunny', '晴朗的'),

            ],
            [
                # ('人教版小学英语四年级下册单词表（三会）Unit 5'),
                ('how much', '多少钱'), ('big', '大的'), ('small', '小的'),
                ('long', '长的'), ('short', '短的'), ('apple', '苹果'),
                ('banana', '香蕉'), ('pear', '梨'), ('orange', '橙子'),
                ('watermelon', '西瓜'), ('are', '就是'), ('they', '它她，她们'),
            ],
            [
                # ('人教版小学英语四年级下册单词表（三会）Unit 6'),
                ('horse', '马'), ('aren\'t', 'are not', '不是'), ('cat', '猫'),
                ('rabbit', '兔子'), ('pig', '猪'), ('duck', '鸭子'),
                ('dog', '狗'),
                ('eleven', '十一'), ('twelve', '十二'), ('thirteen', '十三'),
                ('fifteen', '十五'), ('twenty', '二十'), ('how many', '多少'),
                ('there', '那儿;那里'),
            ]
        ],
        # ('五年级上册词汇'),
        [
            [
                # ('人教版小学英语五年级上册单词表（三会）Unit 1'),
                ('young', '年轻的'), ('funny', '滑稽可笑的'), ('tall', '高的'),
                ('strong', '强壮的'), ('kind', '与蔼的;亲切的'),
                ('old', '年老的'), ('short', '矮的'), ('thin', '瘦的'),
                ('Mr', '先生'), ('like', '像……一样；喜欢'), ('strict', '严格的'),
                ('smart', '聪明的;巧妙的'), ('active', '积极的;活跃的'),
                ('quiet', '安静的;文静的'), ('very', '很;非常'),
                ('but', '但就是'),

            ],
            [
                # ('人教版小学英语五年级上册单词表（三会）Unit 2'),
                ('Mondy', '星期一'), ('Tuesday', '星期二'),
                ('Wednesday', '星期三'), ('Thursday', '星期四'),
                ('Friday', '星期五'), ('Saturday', '星期六'),
                ('Sunday', '星期天'), ('day', '天'), ('have', '有;吃'),
                ('on', '在……时候'), ('do homework', '做作业'),
                ('watch TV', '看电视'), ('read books', '读书'),
            ],
            [
                # ('人教版小学英语五年级上册单词表（三会）Unit 3'),
                ('eggplant', '茄子'), ('fish', '鱼'),
                ('green beans', '青豆'), ('tofu', '豆腐'),
                ('potato', '土豆'), ('tomato', '西红柿'), ('for', '为'),
                ('lunch', '中餐'), ('we', '我们'), ('tasty', '好吃的'),
                ('sweet', '甜的'), ('sour', '酸的'), ('fresh', '新鲜的'),
                ('salty', '咸的'), ('favourite', '最喜欢的'),
                ('they are', '他/她/它们是'), ('fruit', '水果'), ('grape', '葡萄'),
            ],
            [
                # ('人教版小学英语五年级上册单词表（三会）Unit 4'),
                ('cook the meals', '煮饭'), ('water the flowers', '浇花'),
                ('sweep the floor', '扫地'), ('clean the bedroom', '打扫卧室'),
                ('make the bed', '铺床'), ('set the table', '摆饭桌'),
                ('wash the clothes', '洗衣服'),
                ('do the dishes', '洗碗;洗餐具'),
                ('use a computer', '使用计算机'),
            ],
            [
                # ('人教版小学英语五年级上册单词表（三会）Unit 5'),
                ('curtain', '窗帘'), ('trash bin', '垃圾箱'), ('closet', '壁橱'),
                ('mirror', '镜子'), ('bed table', '床头柜'), ('bedroom', '卧室'),
                ('kitchen', '厨房'), ('bathroom', '卫生间'),
                ('living room', '客厅'), ('in', '在……里面'), ('on', '在……上面'),
                ('under', '在……下面'), ('near', '在……旁边'),
                ('behind', '在……后边'), ('clothes', '衣服'),
            ],
            [
                # ('人教版小学英语五年级上册单词表（三会）Unit 6'),
                ('river', '河流'), ('flower', '花'), ('grass', '草'),
                ('lake', '湖泊'), ('forest', '森林'), ('path', '路'),
                ('park', '公园'), ('picture', '照片'), ('house', '房子'),
                ('bridge', '桥'), ('tree', '树'), ('road', '公路'),
                ('building', '建筑物'), ('clean', '干净的'),
            ],
        ],
        # ('五年级下册词汇'),
        [
            [
                # ('人教版小学英语五年级下册单词表（三会）Unit 1'),
                ('eat breakfast', '吃早饭'), ('have English class', '上英语课'),
                ('play sports', '进行体育运动'), ('eat dinner', '吃晚饭'),
                ('when', '什么时候'), ('evening', '夜晚;晚上'),
                ('get up', '起床'),
                ('at', '在……点钟'), ('usually', '通常;一般'),
                ('noon', '中午'), ('climb mountains', 'climb mountain', '爬山'),
                ('go shopping', '购物;买东西'),
                ('play the piano', '弹钢琴'),
                ('visit grandparents', '探望祖父母'),
                ('go hiking', '去远足'), ('weekend', '周末'), ('often', '经常'),
                ('sometimes', '有时候'),

            ],
            [
                # ('人教版小学英语五年级下册单词表（三会）Unit 2'),
                ('spring', '春天'), ('summer', '夏天'), ('fall', '秋天'),
                ('winter', '冬天'), ('season', '季节'), ('which', '哪一个'),
                ('best', '最;极'), ('swim', '游泳'),
                ('fly kites', 'fly kite', '放风筝'), ('skate', '滑冰;滑冰鞋'),
                ('make a snowman', '堆雪人'), ('plant trees', '种树'),
                ('why', '为什么'), ('because', 'cause', '因为'),
                ('sleep', '睡觉'),
            ],
            [
                # ('人教版小学英语五年级下册单词表（三会）Unit 3'),
                ('Jan', 'January', '一月'), ('Feb', 'February', '二月'),
                ('Mar', 'March', '三月'), ('Apr', 'April', '四月'),
                ('May', '五月'), ('June', '六月'), ('July', '七月'),
                ('Aug', 'August', '八月'), ('Sept', 'September', '九月'),
                ('Oct', 'October', '十月'), ('Nov', 'November', '十一月'),
                ('Dec', 'December', '十二月'), ('birthday', '生日'),
                ('uncle', '叔叔;舅舅'), ('her', '她的'), ('date', '日期'),
            ],
            [
                # ('人教版小学英语五年级下册单词表（三会）Unit 4'),
                ('draw pictures', 'draw picture', '画画'),
                ('cook dinner', '做饭'), ('read a book', '看书'),
                ('answer the phone', '接电话'),
                ('listen to music', '听音乐'), ('clean the room', '打扫房间'),
                ('write a letter', '写信'), ('write an e-mail', '写电子邮件'),
                ('mom', '妈妈'), ('grandpa', '爷爷;外公'),
                ('study', '书房'),
            ],
            [
                # ('人教版小学英语五年级下册单词表（三会）Unit 5'),
                ('fly', '飞'), ('jump', '跳'), ('walk', '走'), ('run', '跑'),
                ('swim', '游泳'), ('kangaroo', '袋鼠'), ('sleep', '睡觉'),
                ('climb', '往上爬'), ('fight', '打架'), ('swing', '荡;荡秋千'),
                ('drink water', '喝水'),
            ],
            [
                # ('人教版小学英语五年级下册单词表（三会）Unit 6'),
                ('take pictures', '照相'),
                ('watch insects', 'watch insect', '观察昆虫'),
                ('pick up leave', 'pick up leaves', '采摘树叶'),
                ('do an experiment', '做实验'), ('catch butterfly', '捉蝴蝶'),
                ('honey', '蜂蜜'), ('count insects', '数昆虫'),
                ('collect leaves', '收集树叶'), ('write a report', '写报告'),
                ('play chess', '下棋'), ('have a picnic', '举行野餐'),
            ]
        ],
        [
            # ('六年级上册词汇'),
            [
                # ('人教版小学英语六年级上册单词表（三会）Unit 1'),
                ('by', '通过;乘坐'), ('foot', '脚'), ('bike', '自行车'),
                ('bus', '公共汽车'), ('train', '火车'), ('how', '怎样'),
                ('go to school', '上学'), ('traffic', '交通'),
                ('traffic light', '交通灯'), ('traffic rule', '交通规则'),
                ('stop', '停;停车站'), ('wait', '等待'), ('get to', '到达'),
            ],
            [
                # ('人教版小学英语六年级上册单词表（三会）Unit 2'),
                ('library', '图书馆'), ('post office', '邮局'),
                ('hospital', '医院'), ('cinema', '电影院'), ('bookstore', '书店'),
                ('where', '在哪里;到哪里'), ('please', '请'),
                ('next to', '与……相邻'), ('turn', '转弯'), ('right', '右边'),
                ('left', '左边'), ('straight', '成直线地'), ('then', '然后'),
            ],
            [
                # ('人教版小学英语六年级上册单词表（三会）Unit 3'),
                ('next week', '下周'), ('this morning', '今天上午'),
                ('this afternoon', '今天下午'), ('this evening', '今天晚上'),
                ('comic book', '漫画书'), ('post card', '明信片'),
                ('newspaper', '报纸'), ('buy', '购买'),
            ],
            [
                # ('人教版小学英语六年级上册单词表（三会）Unit 4'),
                ('hobby', '爱好'), ('ride a bike', 'riding a bike', '骑自行车'),
                ('dive', 'diving', '跳水'),
                ('play the violin', 'playing the violin', '拉小提琴'),
                ('make kites', 'making kites', '制作风筝'),
                ('collect stamps', 'collecting stamps', '收集邮票'),
                ('live', 'lives', '居住'), ('teach', 'teaches', '教'),
                ('go', 'goes', '去'), ('watch', 'watches', '瞧'),
                ('read', 'reads', '读'), ('doesn\'t', 'does not', '不做'),
            ],
            [
                # ('人教版小学英语六年级上册单词表（三会）Unit 5'),
                ('singer', '歌唱家;歌手'), ('writer', '作家'),
                ('actor', '男演员'), ('actress', '女演员'),
                ('artist', '画家'), ('TV reporter', '电视台记者'),
                ('engineer', '工程师'), ('accountant', '会计'),
                ('policeman', '男警察'), ('salesperson', '销售员'),
                ('cleaner', '清洁工'), ('where', '在哪里;到哪里'),
                ('work', '工作'),
            ],
            [
                # ('人教版小学英语六年级上册单词表（三会）Unit 6'),
                ('rain', '雨'), ('cloud', '云'), ('sun', '太阳'),
                ('stream', '河;溪'), ('come from', '来自;从……来'),
                ('seed', '种子'), ('soil', '土壤'), ('sprout', '苗;芽'),
                ('plant', '植物;种植'), ('should', '应该'), ('then', '然后'),
            ]
        ],
        [
            # ('六年级下册词汇'),
            [
                # ('人教版小学英语六年级下册单词表（三会）Unit 1'),
                ('taller', '更高的'), ('shorter', '更矮的'),
                ('stronger', '更强壮的'),
                ('older', '年龄更大的'), ('younger', '更年轻的'),
                ('bigger', '更大的'), ('heavier', '更重的'), ('longer', '更长的'),
                ('thinner', '更瘦的'), ('smaller', '体型更小的'),
            ],
            [
                # ('人教版小学英语六年级下册单词表（三会）Unit 2'),
                ('have a fever', '发烧'), ('have a sore throat', '喉咙疼'),
                ('have a cold', '感冒'), ('have a toothache', '牙疼'),
                ('have a headache', '头疼'), ('matter', '事情;麻烦'),
                ('sore', '疼的'), ('hurt', '疼痛'), ('nose', '鼻子'),
                ('tired', '疲劳的;累的'), ('excited', '兴奋的'),
                ('angry', '生气的'), ('happy', '高兴的'),
                ('bored', '无聊的，烦人的'),
                ('sad', '忧伤的，悲伤的'),
            ],
            [
                # ('人教版小学英语六年级下册单词表（三会）Unit 3'),
                ('watched', '瞧'), ('washed', '洗'), ('cleaned', '打扫'),
                ('played', '玩'), ('visited', '瞧望'), ('read', '读'),
                ('did', '进行,做'), ('last weekend', '上一个周末'),
                ('went', '去'), ('went to a park', '去公园'),
                ('went swimming', '去游泳'),
                ('went fishing', '去钓鱼'), ('went hiking', '去郊游'),
            ],
            [
                # ('人教版小学英语六年级下册单词表（三会）Unit 4'),
                ('learned Chinese', '学汉语,学语文'),
                ('sang and danced', '唱歌与跳舞'),
                ('ate good food', '吃好吃的食物'),
                ('took pictures', 'took picture', '照相'), ('climbed', '爬'),
                ('had buy presents', 'bought presents', '买礼物'),
                ('rowed a boat', '划船'), ('saw elephant', '看大象'),
                ('went skiing', '去滑雪'), ('went ice-skating', '去滑冰'),
                ('how', '怎么,如何'), ('got', '到达'),
                ('last', '上一个的，仅余的，留在最后的'),
            ]
        ]
    ]





import copy
import os
import pickle
import random
import sys
from abc import ABC, abstractmethod
from datetime import datetime, timedelta
from typing import Any, List, Optional, Sequence, Text, Tuple, Union, overload

import pygame
import pygame_menu
from pygame.locals import *
from xpinyin import Pinyin

from primaryschool.dirs import *
from primaryschool.locale import _, sys_lang_code
from primaryschool.resource import (default_font, default_font_path,
                                    get_default_font, get_font_path)
from primaryschool.subjects import *


class GameBase(ABC):
    @abstractmethod
    def __init__(self): ...

    @abstractmethod
    def save(self): ...

    @abstractmethod
    def load(self): ...

    @abstractmethod
    def play(self): ...



import getopt
import os
import sys

from primaryschool.settings import *

requirements = \
    [
        # ('requirement_name','version','project_url','License','license_url')
        [
            (  # dev
                'isort', '', 'https://github.com/pycqa/isort',
                'MIT', 'https://github.com/PyCQA/isort/blob/main/LICENSE'
            ),
            (
                'autopep8', '', 'https://github.com/hhatto/autopep8',
                'MIT', 'https://github.com/hhatto/autopep8/blob/master/LICENSE'
            ),
            (
                'nose2', '', 'https://github.com/nose-devs/nose2',
                'BSD License',
                'https://github.com/nose-devs/nose2/blob/main/setup.py#L57'
            ),
            (
                'twine','','https://github.com/pypa/twine/',
                'Apache License 2.0',
                'https://github.com/pypa/twine/blob/main/LICENSE'
            )
        ]
    ] + requirements


def get_requirements_dev():
    install_requires = ''
    for r in requirements:
        for _r in r:
            install_requires += ' ' + _r[0] + _r[1]
    return install_requires


def install_requirements_dev():
    requirements_dev = get_requirements_dev()
    os.system(install_prefix + requirements_dev)


def get_requirements_dev_u():
    install_requires = '-U'
    for r in requirements:
        for _r in r:
            install_requires += ' ' + _r[0]
    return install_requires


def install_requirements_dev_u():
    requirements_dev_u = get_requirements_dev_u()
    os.system(install_prefix + requirements_dev_u)


argv = sys.argv[1:]

for arg in argv:
    if arg == 'req_dev':
        install_requirements_dev()

    if arg == 'req_dev_u':
        install_requirements_dev_u()

    if arg == 'test':
        import tests

if len(argv) == 0:
    import re
    import sys

    import tests
    from primaryschool import victory
    if __name__ == '__main__':
        sys.argv[0] = re.sub(r'(-script\.pyw|\.exe)?$', '', sys.argv[0])
        sys.exit(victory())




import os
import sys

app_name = 'primaryschool'
app_version = '0.0.19'
app_author = 'larryw3i'
app_author_email = 'larryw3i@163.com'
app_maintainer = app_author
app_maintainer_email = app_author_email
app_description = "primary school knowledge games"
app_url = "https://github.com/larryw3i/primaryschool"
app_contributors = [
    app_author,
    '',
]

app_sponsors = [
    ''
]

install_prefix = 'python -m pip install '
requirements = [
    # product
    [  # ('requirement_name','version','project_url','License','license_url')
        (
            'pygame', '', 'https://github.com/pygame/pygame',
            'LGPL v2',
            'https://github.com/pygame/pygame/blob/main/docs/LGPL.txt'
        ),
        (
            'pygame-menu', '', 'https://github.com/ppizarror/pygame-menu',
            'MIT',
            'hhttps://github.com/ppizarror/pygame-menu/blob/master/LICENSE'
        ),
        (
            'xpinyin', '', 'https://github.com/lxneng/xpinyin',
            'BSD',
            'https://github.com/lxneng/xpinyin/blob/master/setup.py#L37'
        ),
        (
            'appdirs', '', 'https://github.com/ActiveState/appdirs',
            'MIT',
            'https://github.com/ActiveState/appdirs/blob/master/LICENSE.txt'
        ),
    ],
]


def get_requirements_product():
    install_requires = []
    for r in requirements[0]:
        install_requires.append(r[0] + r[1])
    return install_requires


def install_requirements_product():
    requirements_product = get_requirements_product()
    os.system(install_prefix + requirements_product)



import copy
import os
import pickle
import random
import sys
from datetime import datetime, timedelta
from typing import Any, List, Optional, Sequence, Text, Tuple, Union, overload

import pygame
import pygame_menu
from pygame.locals import *
from xpinyin import Pinyin

from primaryschool.dirs import *
from primaryschool.locale import _
from primaryschool.resource import (default_font, default_font_path,
                                    get_default_font, get_font_path,
                                    get_resource_path)
from primaryschool.subjects import *
from primaryschool.subjects._abc_ import GameBase
from primaryschool.subjects._common_.shootingbase import *
from primaryschool.subjects.yuwen.words import cn_ps_c, cn_ps_c_bb

# primaryschool.subjects.yuwen.g_pinyin_missile
module_str = __name__

name_t = _('Pinyin Missile')
default_difficulty_index = 16
difficulties = [
    _('Grade 1.1'),  # 0
    _('Grade 1.2'),  # 1
    _('Grade 2.1'),  # 2
    _('Grade 2.2'),  # 3
    _('Grade 3.1'),  # 4
    _('Grade 3.2'),  # 5
    _('Grade 4.1'),  # 6
    _('Grade 4.2'),  # 7
    _('Grade 5.1'),  # 8
    _('Grade 5.2'),  # 9
    _('Grade 6.1'),  # 10
    _('Grade 6.2'),  # 11
    _('Low level'),  # 12
    _('High level'),  # 13
    _('All grades'),  # 14
    _('All Chinese characters'),  # 15
    _('Grade 1.1 (30 characters)'),  # 16
    _('Grade 1.2 (30 characters)'),  # 17
    _('Grade 2.1 (30 characters)'),  # 18
    _('Grade 2.2 (30 characters)'),  # 19
    _('Grade 3.1 (30 characters)'),  # 20
    _('Grade 3.2 (30 characters)'),  # 21
    _('Grade 4.1 (30 characters)'),  # 22
    _('Grade 4.2 (30 characters)'),  # 23
    _('Grade 5.1 (30 characters)'),  # 24
    _('Grade 5.2 (30 characters)'),  # 25
    _('Grade 6.1 (30 characters)'),  # 26
    _('Grade 6.2 (30 characters)'),  # 27
    _('Low level (30 characters)'),  # 28
    _('High level (30 characters)'),  # 29
    _('All grades (30 characters)'),  # 30
    _('All Chinese characters (30 characters)'),  # 31

]

help_t = _('''
Enter the pinyin corresponding to the Chinese character, and enter the number
after the pinyin to indicate the tone.
''')

pinyin = Pinyin()

cn_ps_chars = cn_ps_c_bb


class PmTargetsManager(TargetsManager):
    def __init__(self, shtbase):
        super().__init__(
            shtbase,
            target_surface_lang_code='zh_CN'
        )
        self.wave = ShootingWave(self)

    def blit_intercepting(self, moving_surfaces):
        self.wave.draw(moving_surfaces.intercept_frame_counter)

    def get_targets(self, d: int = 0, count=30):

        targets = None
        _base_len = int(len(difficulties) / 2)
        g_index = d
        if g_index < _base_len:
            if g_index == _base_len - 1:
                targets = self.get_rand_words(self.rand_word_count)
            if 0 <= g_index < _base_len - 1:
                targets = self.get_cn_ps_words(g_index)
        else:
            g_index = g_index - _base_len
            if g_index == _base_len - 1:
                targets = random.choices(
                    self.get_rand_words(self.rand_word_count), k=30)
            if 0 <= g_index < _base_len - 1:
                targets = random.choices(
                    self.get_cn_ps_words(g_index), k=30)

        _targets = []
        for t in targets:
            p = pinyin.get_pinyins(t, tone_marks='numbers')
            p.append(t)
            _targets.append(tuple(p))

        return _targets

    def get_cn_ps_words(self, g_index: int):
        words = []
        if g_index < 12:
            words = cn_ps_chars[g_index]
        elif g_index == 12:
            words = sum(cn_ps_chars[0:6], [])
        elif g_index == 13:
            words = sum(cn_ps_chars[6:16], [])
        elif g_index == 14:
            words = sum(cn_ps_chars[0:16], [])
        return sum(words, [])

    def get_rand_words(self, n):
        return [chr(random.randint(0x4e00, 0x9fbf)) for i in range(0, n)]


class PinyinMissile(ShootingBase):
    def __init__(self, ps):
        self.name_t = name_t
        self.difficulties = difficulties
        self.module_str = module_str
        super().__init__(ps)

    def get_targets_manager(self):
        return PmTargetsManager(self)

    def key_clean(self, code):
        return \
            self.keycode_in_pure_num(code) or \
            self.keycode_in_alpha(code)


def enjoy(ps):
    return PinyinMissile(ps)




import os
import re

import pkg_resources
import setuptools

from primaryschool.settings import *

long_description = open("README.md", "r", encoding="utf-8").read()

setuptools.setup(
    name=app_name,
    version=app_version,
    author=app_author,
    author_email=app_author_email,
    maintainer=app_maintainer,
    maintainer_email=app_maintainer_email,
    description=app_description,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=app_url,
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    entry_points={
        'console_scripts': [
            'primaryschool=primaryschool:victory',
        ]
    },
    python_requires='>=3.6',
    install_requires=get_requirements_product(),
    include_package_data=True,
)



import copy
import os
import pickle
import random
import sys
from datetime import datetime, timedelta
from operator import *
from typing import Any, List, Optional, Sequence, Text, Tuple, Union, overload

import pygame
import pygame_menu
from pygame.locals import *

from primaryschool.dirs import *
from primaryschool.locale import _, sys_lang_code
from primaryschool.resource import (default_font, default_font_path,
                                    get_default_font, get_font_path)
from primaryschool.subjects import *
from primaryschool.subjects._abc_ import GameBase
from primaryschool.subjects._common_.shootingbase import *

# primaryschool.subjects.yuwen.g_mind_hunter
module_str = __name__

name_t = _('Mind hunter')

difficulties = [
    _('< 10 + 10'),  # 0
    _('< 50 + 50'),  # 1
    _('< 100 + 100'),  # 2
    _('< 10 - 10'),  # 3
    _('< 50 - 50'),  # 4
    _('< 100 - 100'),  # 5
    _('< 10 * 10'),  # 6
    _('< 50 * 50'),  # 7
    _('< 100 * 100'),  # 8
    _('< 10 / 10'),  # 9
    _('< 50 / 50'),  # 10
    _('< 100 / 100'),  # 11
    _('< 10 ? 10'),  # 12
    _('< 50 ? 50'),  # 13
    _('< 100 ? 100'),  # 14
]


help_t = _('''
Enter the calculation result, press Enter to confirm.
''')

times_sign = '\u00d7'
division_sign = '\u00f7'


class MhTargetsManager(TargetsManager):
    def __init__(self, shtbase):
        super().__init__(
            shtbase,
            intercept_keycode=0xd
        )
        self.wave = ShootingWave(self)

    def get_pmt_formulas(self, _max, _oper, count=None):  # for plus/minus/time
        f = []
        count = count if count else self.target_count
        _max_sqrt = int(_max**0.5)
        for _ in range(self.target_count + 1):
            _oper_ = random.choice(_oper) if isinstance(_oper, list) else _oper
            num0 = random.randint(0, _max)
            num1 = random.randint(0, _max)
            if _oper_ == '-':
                num0 = max(num0, num1)
                num1 = min(num0, num1)
            if _oper_ == times_sign:
                num0 = random.randint(0, _max_sqrt)
                num1 = random.randint(0, _max_sqrt)
            f.append(str(num0) + ' ' + _oper_ + ' ' + str(num1))

        return f

    def blit_intercepting(self, moving_surfaces):
        self.wave.draw(moving_surfaces.intercept_frame_counter)

    def get_division_formulas(self, _max, count=None):
        _d = []
        count = count if count else self.target_count
        for i in range(_max + 1):
            for j in range(1, _max + 1):
                if i % j == 0:
                    _d.append((i, j))
        return [str(a) + ' ' + division_sign + ' ' + str(b)
                for a, b in random.choices(_d, k=self.target_count)]

    def get_result(self, formula):
        a, oper, b = formula.split(' ')
        return str(
            int(a) + int(b) if oper == '+' else
            int(a) - int(b) if oper == '-' else
            int(a) * int(b) if oper == times_sign else
            int(int(a) / int(b))
        )

    def get_targets(self, d: int = 0, count=20):
        formulas = None
        if d < 15:
            _rem, _quo = d % 3, d // 3
            _max = \
                10 if _rem == 0 else \
                50 if _rem == 1 else \
                100
            _oper = \
                '+' if _quo == 0 else \
                '-' if _quo == 1 else \
                times_sign if _quo == 2 else \
                division_sign if _quo == 3 else \
                '?'
            if _oper in ['+', '-', times_sign]:
                formulas = self.get_pmt_formulas(_max, _oper)
            elif _oper == division_sign:
                formulas = self.get_division_formulas(_max)
            else:  # ?
                _d_opers_count = random.choices(
                    ['+', '-', times_sign, division_sign],
                    k=self.target_count)\
                    .count(division_sign)
                formulas = \
                    self.get_pmt_formulas(_max, ['+', '-', times_sign]) + \
                    self.get_division_formulas(_max, _d_opers_count)

            return [(self.get_result(f), f) for f in formulas]


class MindHunter(ShootingBase):
    def __init__(self, ps):
        self.name_t = name_t
        self.difficulties = difficulties
        self.module_str = module_str
        super().__init__(ps)

    def get_targets_manager(self):
        return MhTargetsManager(self)

    def key_clean(self, code):
        return \
            self.keycode_in_return(code) or \
            self.keycode_in_num(code)


def enjoy(ps):
    return MindHunter(ps)




import os
import sys

import pygame

from primaryschool import project_path
from primaryschool.locale import _, sys_lang_code

resource_dir_path = os.path.abspath(os.path.dirname(__file__))
project_font_dir_path = os.path.join(resource_dir_path, 'fonts')

pygame.font.init()


class Resource():
    def __init__(self):

        self.default_font_size = 50
        self.sys_font_names = pygame.font.get_fonts()
        self.locale_font_paths = self.get_locale_font_paths()
        self.resource_dir_names = ['imgs', 'audios', 'fonts']
        self.resource_paths = self.get_resource_paths()

    def get_sys_font_name_like(self, _like_name):
        for f in self.sys_font_names:
            if _like_name.lower() in f.lower():
                return f
        return self.sys_font_names[0]

    def get_font_path(self, lang_code='', show_not_found=False):
        lang_code = sys_lang_code if len(lang_code) < 1 else lang_code
        for k, v in self.locale_font_paths.items():
            if lang_code == k:
                return v

        if show_not_found:
            from tkinter import Tk, messagebox
            root = Tk()
            messagebox.showerror(
                _('No font found'),
                _('Could not find font of %s.') % lang_code)
            root.destroy()

        return self.locale_font_paths['default']

    def get_resource_paths(self):
        resource_paths = []
        for root, dirs, files in os.walk(resource_dir_path, topdown=False):
            for n in self.resource_dir_names:
                # imgs/xx_XX/xx.xx, for locale resources.
                if (root.endswith(n) or root.split(os.sep)[-2] == n) \
                        and len(dirs) < 1:
                    for name in files:
                        resource_paths.append(os.path.join(root, name))
        return sorted(resource_paths, key=len)

    def get_resource_path(self, name):
        assert os.sep not in name
        for f in self.resource_paths:
            if f.endswith(name):
                return f

    def get_locale_font_paths(self):
        return \
            {
                'default': pygame.font.match_font(
                    self.get_sys_font_name_like('mono')),
                'zh_CN': pygame.font.match_font(
                    self.get_sys_font_name_like(
                        sys.platform == 'darwin' and 'heiti' or
                        sys.platform == 'win32' and 'yahei' or
                        'cjk'
                    )
                ),
            }


r = Resource()
default_font_path = r.get_font_path()


def get_default_font(size=None):
    return pygame.font.Font(
        default_font_path,
        size or r.default_font_size)


def get_font_path(
        lang_code,
        show_not_found=False):

    return r.get_font_path(lang_code, show_not_found)


def get_sys_font_by_lang_code(
        lang_code=None,
        size=None):

    size = size or r.default_font_size
    return \
        pygame.font.Font(get_font_path(lang_code), size) \
        if lang_code else \
        get_default_font()


def get_resource_path(name):
    return r.get_resource_path(name)


default_font = get_default_font()





import importlib
import os

from appdirs import AppDirs

from primaryschool.settings import app_author, app_name, app_version

_dirs = AppDirs(app_name, app_author)

user_data_dir_path = _dirs.user_data_dir
user_cache_dir_path = _dirs.user_cache_dir
user_log_dir_path = _dirs.user_log_dir
user_config_dir_path = _dirs.user_config_dir

for d in [
        user_data_dir_path, user_cache_dir_path,
        user_log_dir_path, user_config_dir_path]:
    if not os.path.exists(d):
        os.makedirs(d, exist_ok=True)


def get_copy_path(module_str):
    return os.path.join(user_data_dir_path, module_str) + f'.{app_version}.pkl'
